@@ -4,7 +4,10 @@ function(
 	, dv
 	, wid
 	, within = NULL
+	, within_full = NULL
+	, within_covariates = NULL
 	, between = NULL
+	, between_covariates = NULL
 	, observed = NULL
 	, diff = NULL
 	, reverse_diff = FALSE
@@ -13,13 +16,34 @@ function(
 	, detailed = FALSE
 	, return_aov = FALSE
 ){
+	args_to_check = c('dv','wid','within','between','observed','diff','within_full','within_covariates','between_covariates')
+	args = as.list(match.call()[-1])
+	for(i in 1:length(args)){
+		arg_name = names(args)[i]
+		if(arg_name%in%args_to_check){
+			if(is.symbol(args[[i]])){
+				code = paste(arg_name,'=.(',as.character(args[[i]]),')',sep='')
+				eval(parse(text=code))
+			}else{
+				if(is.language(args[[i]])){
+					arg_vals = as.character(args[[i]])
+					arg_vals = arg_vals[2:length(arg_vals)]
+					arg_vals = paste(arg_vals,collapse=',')
+					code = paste(arg_name,'=.(',arg_vals,')',sep='')
+					eval(parse(text=code))
+				}
+			}
+		}
+	}
 	to_return = ezANOVA_main(
 		data = data
 		, dv = dv
 		, wid = wid
 		, within = within
+		, within_full = within_full
+		, within_covariates = within_covariates
 		, between = between
-		, observed = observed
+		, between_covariates = between_covariates
 		, diff = diff
 		, reverse_diff = reverse_diff
 		, type = type
