@@ -5,9 +5,29 @@ function(
 	, wid
 	, within = NULL
 	, between = NULL
-	, perms
-	, alarm = TRUE
+	, perms = 1e3
+	, parallel = FALSE
+	, alarm = FALSE
 ){
+	args_to_check = c('dv','wid','within','between')
+	args = as.list(match.call()[-1])
+	for(i in 1:length(args)){
+		arg_name = names(args)[i]
+		if(arg_name%in%args_to_check){
+			if(is.symbol(args[[i]])){
+				code = paste(arg_name,'=.(',as.character(args[[i]]),')',sep='')
+				eval(parse(text=code))
+			}else{
+				if(is.language(args[[i]])){
+					arg_vals = as.character(args[[i]])
+					arg_vals = arg_vals[2:length(arg_vals)]
+					arg_vals = paste(arg_vals,collapse=',')
+					code = paste(arg_name,'=.(',arg_vals,')',sep='')
+					eval(parse(text=code))
+				}
+			}
+		}
+	}
 	if(is.null(within) & is.null(between)){
 		stop('is.null(within) & is.null(between)\nYou must specify at least one independent variable.')
 	}
@@ -45,8 +65,6 @@ function(
 		}
 	}
 	data=data[,names(data) %in% c(within,between,wid,dv)]
-	cat('Permutation test progress:\n')
-	start = proc.time()[3]
 	aov_formula = paste(
 		as.character(dv)
 		,'~'
@@ -73,7 +91,6 @@ function(
 		,sep = ''
 	)	
 	obs = ezPerm_aov(data,aov_formula)
-	sim = matrix(NA,length(obs),perms)
 	sim_data=data
 	if(!is.null(between)){
 		group_info = ddply(
@@ -84,29 +101,32 @@ function(
 			}
 		)
 	}
-	progress = create_progress_bar('timeCI')
-	progress$init(perms)
-	for(this_perm in 1:perms){
-		for(this_within in within){
-			sim_data = ddply(
-				sim_data
-				,.variables = structure(as.list(c(wid,structure(within[within!=this_within],class='quoted'))),class='quoted')
-				,function(x){
-					x[,names(x)==this_within] = sample(x[,names(x)==this_within])
-					return(x)
-				}
-			)
-		}
-		for(this_between in between){
-			group_info[,names(group_info)==this_between]=sample(group_info[,names(group_info)==this_between])
-			for(this_wid in sim_data[,names(sim_data)==wid]){
-				sim_data[sim_data[,names(sim_data)==wid]==this_wid,names(sim_data)==this_between] = group_info[group_info[,names(group_info)==wid]==this_wid,names(group_info)==this_between]
+	sim = laply(
+		.data = 1:perms
+		, .fun = function(this_perm){
+			set.seed(this_perm)
+			for(this_within in within){
+				sim_data = ddply(
+					sim_data
+					,.variables = structure(as.list(c(wid,structure(within[within!=this_within],class='quoted'))),class='quoted')
+					,function(x){
+						x[,names(x)==this_within] = sample(x[,names(x)==this_within])
+						return(x)
+					}
+				)
 			}
+			for(this_between in between){
+				group_info[,names(group_info)==this_between]=sample(group_info[,names(group_info)==this_between])
+				for(this_wid in sim_data[,names(sim_data)==wid]){
+					sim_data[sim_data[,names(sim_data)==wid]==this_wid,names(sim_data)==this_between] = group_info[group_info[,names(group_info)==wid]==this_wid,names(group_info)==this_between]
+				}
+			}
+			return(ezPerm_aov(sim_data,aov_formula))
 		}
-		sim[,this_perm] = ezPerm_aov(sim_data,aov_formula)
-		progress$step()
-	}
-	progress$term()
+		, .progress = 'time'
+		, .parallel = parallel
+	)
+	sim = matrix(sim,ncol=perms)
 	from_terms = terms(eval(parse(text=aov_formula)))
 	term_labels = attr(from_terms,'term.labels')
 	term_labels = term_labels[grep('(Intercept)',term_labels,fixed=T,invert=T)]
@@ -114,7 +134,9 @@ function(
 	perm_test = data.frame(Effect=term_labels)
 	perm_test$p = rowMeans(sim>=obs)
 	perm_test$'p<.05' = ifelse(perm_test$p<.05,'*','')
-	alarm()
+	if(alarm){
+		alarm()
+	}
 	return(perm_test)
 }
 
